#include <stdlib.h>
#include <string.h>
#include "newsraft.h"

static inline bool
append_sorting_order_expression_to_query(struct string *q, int order)
{
	switch (order) {
		case SORT_BY_TIME_ASC:              catas(q, " ORDER BY MAX(publication_date, update_date) ASC, rowid ASC", 59); break;
		case SORT_BY_TIME_DESC:             catas(q, " ORDER BY MAX(publication_date, update_date) DESC, rowid DESC", 61); break;
		case SORT_BY_TIME_UPDATE_ASC:       catas(q, " ORDER BY update_date ASC, rowid ASC", 36); break;
		case SORT_BY_TIME_UPDATE_DESC:      catas(q, " ORDER BY update_date DESC, rowid DESC", 38); break;
		case SORT_BY_TIME_PUBLICATION_ASC:  catas(q, " ORDER BY publication_date ASC, rowid ASC", 41); break;
		case SORT_BY_TIME_PUBLICATION_DESC: catas(q, " ORDER BY publication_date DESC, rowid DESC", 43); break;
		case SORT_BY_ROWID_ASC:             catas(q, " ORDER BY rowid ASC", 19); break;
		case SORT_BY_ROWID_DESC:            catas(q, " ORDER BY rowid DESC", 20); break;
		case SORT_BY_UNREAD_ASC:            catas(q, " ORDER BY unread ASC, MAX(publication_date, update_date) DESC, rowid DESC", 73); break;
		case SORT_BY_UNREAD_DESC:           catas(q, " ORDER BY unread DESC, MAX(publication_date, update_date) DESC, rowid DESC", 74); break;
		case SORT_BY_IMPORTANT_ASC:         catas(q, " ORDER BY important ASC, MAX(publication_date, update_date) DESC, rowid DESC", 76); break;
		case SORT_BY_IMPORTANT_DESC:        catas(q, " ORDER BY important DESC, MAX(publication_date, update_date) DESC, rowid DESC", 77); break;
		case SORT_BY_ALPHABET_ASC:          catas(q, " ORDER BY title ASC, rowid ASC", 30); break;
		case SORT_BY_ALPHABET_DESC:         catas(q, " ORDER BY title DESC, rowid DESC", 32); break;
		default:
			fail_status("Unknown sorting method name!");
			return false;
	}
	return true;
}

static inline struct string *
generate_search_query_string(const struct items_list *items)
{
	struct string *query = crtas("SELECT rowid,feed_url,guid,title,link,publication_date,update_date,unread,important FROM items WHERE ", 101);
	struct string *cond = generate_items_search_condition(items->feeds, items->feeds_count);
	if (!STRING_IS_EMPTY(cond)) {
		catss(query, cond);
	}
	free_string(cond);
	if (!STRING_IS_EMPTY(items->find_filter)) {
		catas(query, " AND (", 6);
		catss(query, items->find_filter);
		catcs(query, ')');
	}
	if (!STRING_IS_EMPTY(items->search_filter)) {
		catas(query, " AND ((title LIKE '%' || ? || '%') OR (content LIKE '%' || ? || '%'))", 69);
	}
	if (append_sorting_order_expression_to_query(query, items->sorting) == false) {
		free_string(query);
		return NULL;
	}
	return query;
}

void
free_items_list(struct items_list *items)
{
	if (items != NULL) {
		for (size_t i = 0; i < items->len; ++i) {
			free_string(items->ptr[i].guid);
			free_string(items->ptr[i].title);
			free_string(items->ptr[i].url);
			free_string(items->ptr[i].date_str);
			free_string(items->ptr[i].pub_date_str);
		}
		sqlite3_finalize(items->res);
		free_string(items->query);
		free_string(items->find_filter);
		free_string(items->search_filter);
		free(items->ptr);
		free(items);
	}
}

static inline struct feed_entry **
find_feed_entry_by_url(struct feed_entry **feeds, size_t feeds_count, const char *feed_url)
{
	if (feed_url != NULL) {
		for (size_t i = 0; i < feeds_count; ++i) {
			if (strcmp(feed_url, feeds[i]->link->ptr) == 0) {
				return feeds + i;
			}
		}
	}
	return NULL;
}

void
obtain_items_at_least_up_to_the_given_index(struct items_list *items, size_t index)
{
	if (items->finished == true) {
		return;
	}
	const char *text;
	int status;
	while (index >= items->len) {
		status = sqlite3_step(items->res);
		if (status != SQLITE_ROW) {
			items->finished = true;
			return;
		}

		items->ptr = newsraft_realloc(items->ptr, sizeof(struct item_entry) * (items->len + 1));

		items->ptr[items->len].rowid = sqlite3_column_int64(items->res, 0);

		text = (const char *)sqlite3_column_text(items->res, 1);
		items->ptr[items->len].feed = find_feed_entry_by_url(items->feeds, items->feeds_count, text);
		if (items->ptr[items->len].feed == NULL) {
			continue;
		}

		text = (const char *)sqlite3_column_text(items->res, 2);
		if (text == NULL) {
			continue;
		}
		items->ptr[items->len].guid = crtas(text, strlen(text));

		text = (const char *)sqlite3_column_text(items->res, 3);
		items->ptr[items->len].title = text == NULL ? crtes(1) : crtas(text, strlen(text));
		inlinefy_string(items->ptr[items->len].title);

		text = (const char *)sqlite3_column_text(items->res, 4);
		items->ptr[items->len].url = crtes(1);
		// Convert URL to absolute notation in case it's stored relative.
		// For example, convert "/index.xml" to "http://example.org/index.xml"
		char *full_url = complete_url(items->ptr[items->len].feed[0]->link->ptr, text);
		if (full_url != NULL) {
			cpyas(&items->ptr[items->len].url, full_url, strlen(full_url));
			free(full_url);
		}

		items->ptr[items->len].pub_date = sqlite3_column_int64(items->res, 5);
		items->ptr[items->len].upd_date = sqlite3_column_int64(items->res, 6);
		if (items->ptr[items->len].pub_date <= 0 && items->ptr[items->len].upd_date > 0) {
			items->ptr[items->len].pub_date = items->ptr[items->len].upd_date;
		}
		if (items->ptr[items->len].upd_date <= 0 && items->ptr[items->len].pub_date > 0) {
			items->ptr[items->len].upd_date = items->ptr[items->len].pub_date;
		}
		items->ptr[items->len].date_str = get_cfg_date(NULL, CFG_LIST_ENTRY_DATE_FORMAT, items->ptr[items->len].upd_date);
		items->ptr[items->len].pub_date_str = get_cfg_date(NULL, CFG_LIST_ENTRY_DATE_FORMAT, items->ptr[items->len].pub_date);

		items->ptr[items->len].is_unread = sqlite3_column_int(items->res, 7); // unread
		items->ptr[items->len].is_important = sqlite3_column_int(items->res, 8); // important

		items->len += 1;
	}
}

struct items_list *
create_items_list(struct feed_entry **feeds, size_t feeds_count, const struct wstring *find_filter, const struct items_list *previous_items)
{
	INFO("Generating items list.");
	if (feeds_count == 0) {
		return NULL;
	}

	struct items_list *items = newsraft_calloc(1, sizeof(struct items_list));

	if (previous_items) {
		items->sorting = previous_items->sorting;
	} else {
		items->sorting = get_sorting_id(get_cfg_string(NULL, CFG_MENU_ITEM_SORTING)->ptr);
	}

	if (previous_items && !STRING_IS_EMPTY(previous_items->find_filter)) {
		cpyss(&items->find_filter, previous_items->find_filter);
	} else if (find_filter) {
		items->find_filter = convert_wstring_to_string(find_filter);
	}

	if (previous_items && !STRING_IS_EMPTY(previous_items->search_filter)) {
		cpyss(&items->search_filter, previous_items->search_filter);
	} else {
		items->search_filter = pop_search_filter();
	}

	items->feeds = feeds;
	items->feeds_count = feeds_count;
	items->query = generate_search_query_string(items);
	if (items->query == NULL) {
		goto undo1;
	}
	items->res = db_prepare(items->query->ptr, items->query->len + 1, NULL);
	if (items->res == NULL) {
		fail_status("Can't run items search query! Make sure all item-rule settings are valid SQL conditions.");
		goto undo2;
	}
	for (size_t i = 0; i < feeds_count; ++i) {
		db_bind_string(items->res, i + 1, feeds[i]->link);
	}
	if (!STRING_IS_EMPTY(items->search_filter)) {
		db_bind_string(items->res, feeds_count + 1, items->search_filter);
		db_bind_string(items->res, feeds_count + 2, items->search_filter);
	}
	obtain_items_at_least_up_to_the_given_index(items, 0);
	if (items->len < 1) {
		if (!STRING_IS_EMPTY(items->search_filter)) {
			info_status("No items found. Search query didn't get any matches!");
		} else if (!STRING_IS_EMPTY(items->find_filter)) {
			info_status("No items found. Find query didn't get any matches!");
		} else {
			fail_status("No items found. Make sure this feed is updated!");
		}
		goto undo3;
	}
	return items;
undo3:
	sqlite3_finalize(items->res);
undo2:
	free_string(items->query);
undo1:
	free_string(items->find_filter);
	free_string(items->search_filter);
	free(items);
	return NULL;
}

bool
recreate_items_list(struct items_list **items)
{
	struct items_list *new_items = create_items_list((*items)->feeds, (*items)->feeds_count, NULL, *items);
	if (new_items == NULL) {
		return false;
	}
	pthread_mutex_lock(&interface_lock);
	free_items_list(*items);
	*items = new_items;
	reset_list_menu_unprotected();
	pthread_mutex_unlock(&interface_lock);
	return true;
}

void
change_items_list_sorting(struct items_list **items, input_id cmd)
{
	static const struct { int primary; int secondary; } sort_map[] = {
		[INPUT_SORT_BY_TIME]             = {SORT_BY_TIME_DESC,             SORT_BY_TIME_ASC},
		[INPUT_SORT_BY_TIME_UPDATE]      = {SORT_BY_TIME_UPDATE_DESC,      SORT_BY_TIME_UPDATE_ASC},
		[INPUT_SORT_BY_TIME_PUBLICATION] = {SORT_BY_TIME_PUBLICATION_DESC, SORT_BY_TIME_PUBLICATION_ASC},
		[INPUT_SORT_BY_ROWID]            = {SORT_BY_ROWID_DESC,            SORT_BY_ROWID_ASC},
		[INPUT_SORT_BY_UNREAD]           = {SORT_BY_UNREAD_DESC,           SORT_BY_UNREAD_ASC},
		[INPUT_SORT_BY_ALPHABET]         = {SORT_BY_ALPHABET_ASC,          SORT_BY_ALPHABET_DESC},
		[INPUT_SORT_BY_IMPORTANT]        = {SORT_BY_IMPORTANT_DESC,        SORT_BY_IMPORTANT_ASC},
	};
	(*items)->sorting = (*items)->sorting == sort_map[cmd].primary ? sort_map[cmd].secondary : sort_map[cmd].primary;
	recreate_items_list(items);
	info_status(get_sorting_message((*items)->sorting), "items");
}
